document.body.style.display = '';

var OA_output = [];

// Google Analytics
(function (i, s, o, g, r, a, m) {
  i['GoogleAnalyticsObject'] = r;
  i[r] = i[r] || function () {
      (i[r].q = i[r].q || []).push(arguments)
    }, i[r].l = 1 * new Date();
  a = s.createElement(o),
    m = s.getElementsByTagName(o)[0];
  a.async = 1;
  a.src = g;
  m.parentNode.insertBefore(a, m)
})(window, document, 'script', 'https://www.google-analytics.com/analytics.js', 'ga');

(function () {

  var details = window.muzli.getDetails();

  ga('create', window.GA_TRACKING_CODE, 'auto');

  // Removes failing protocol check. @see: http://stackoverflow.com/a/22152353/1958200
  ga('set', 'checkProtocolTask', function () {});

  if (details) {
    ga('set', 'dimension1', details.version);
  }

  if (localStorage.allowAnonymous) {
    ga('set', 'dimension7', localStorage.allowAnonymous === 'true' ? 'anonymous' : 'non-anonymous');
  }

  ga('require', 'displayfeatures');

  if (window.REGISTERED === 'ftx') {
    ga('send', {
      'hitType': 'pageview',
      'page': '/ftx/login-success',
      'title': 'Login successful',
    });
  } else if (window.REGISTERED) {
    ga('send', {
      'hitType': 'pageview',
      'page': '/index.html',
    });
  } else {
    ga('send', {
      'hitType': 'pageview',
      'page': '/index.html',
      'sessionControl': 'start',
    });
  }

  String.prototype.capitalize = function() {
    return this.charAt(0).toUpperCase() + this.slice(1);
  }

})();

(function () {

  window.muzliOpenWindow = function (url, title) {
    var w = 600;
    var h = 450;
    var left = (screen.width / 2) - (w / 2);
    var top = (screen.height / 2) - (h / 2);
    var win = window.open(url, title, 'toolbar=no, location=no, directories=no, status=no, menubar=no, scrollbars=no, resizable=no, copyhistory=no, width=' + w + ', height=' + h + ', top=' + top + ', left=' + left);
    if (window.focus) {
      win.focus()
    }
  };

  window.debounce = function (func, wait, immediate) {

    var timeout;

    return function () {

        var context = this;
        var args = arguments;

        var later = function () {
            timeout = null;
            if (!immediate) func.apply(context, args);
        };

        var callNow = immediate && !timeout;

        clearTimeout(timeout);

        timeout = setTimeout(later, wait);

        if (callNow) func.apply(context, args);
    };
  }

  var contactLinks = [
    {href: 'https://muz.li/contact/#feedback', text: 'General feedback'},
    {href: 'https://muz.li/contact/#suggest', text: 'Suggest a link'},
    {href: 'https://muz.li/subscribe/', text: 'Subscribe to our weekly digest'},
    {href: 'https://muz.li/contact/#report', text: 'Report a bug'},
    {href: 'https://www.invisionapp.com/privacy', text: 'Privacy Policy'},
    {href: 'https://www.invisionapp.com/terms', text: 'Terms of Service'}
  ];

  async function fetchMuzli(feedFetcher, sort) {

    //Show top week Muzli posts for FTX users (Until they scroll)
    if (localStorage.fetchLastWeek) {
      
      let topWeek = await feedFetcher.fetch('top-week', sort, 15);
      
      addShareLinks(topWeek.data);
      
      return Promise.resolve(topWeek.data);
      
    }
    
    return feedFetcher.fetch('muzli', sort, 15)
    .then(async (res) => {

        let seenPinnedItems = JSON.parse(localStorage.getItem('seenPinnedItems') || '{}');

        //Remove pinned flag from items that were seen too many times
        res.data.forEach(item => {
          if (item.isPinned && seenPinnedItems[item.id] >= 7) {
            delete item.isPinned;
          }
        })

        //Move pinned items to front
        res.data.sort((a, b) => {
          return a.isPinned ? -1 : b.isPinned ? 1 : 0;
        })

        addShareLinks(res.data);

        return res.data;

      });
  }

  async function fetchDynamicFeed(feedFetcher, $rootScope) {
    
    $rootScope.feedVariation = parseInt(localStorage.dynamicSlot) || 0; 

    //Trending feed from top clicked items
    if ($rootScope.feedVariation === 0) {

      localStorage['dynamicSlot'] = 1;
      
      return feedFetcher
      .fetch('all:trending', 'clicks', 15, 'top')
      .then(function (res) {
        return res.data.filter(item => item.source !== 'muzli');
      });

    }

    //Personalized all feed sorted by virality
    if ($rootScope.feedVariation > 0) {

      localStorage['dynamicSlot'] = 0;

      return feedFetcher
      .fetch('all:viral', 'virality', 15)
      .then(function (res) {
        return res.data.filter(item => item.source !== 'muzli');
      });
    }

  }

  function fetchAll(feedFetcher, sort, filter) {

    let limit = window.muzli.paging.server

    return feedFetcher
    .fetch(null, sort, limit, filter)
    .then(function (res) {

      addShareLinks(res.data);

      return res.data.map(function (item) {
        item.pick = item.source && item.source.name === 'muzli';
        return item;
      });

    });
  }

  function fetchMuzliClicked(storage) {
    return storage.get("postClicks", true).then(function (obj) {
      return (obj && obj.postClicks) || [];
    });
  }

  function synchMuzliClicked(storage, clickedItems) {
    storage.set({
      postClicks: (clickedItems || []).slice(-500)
    })
  }

  function setAllFeed($timeout, $scope, data) {

    $timeout(function () {

      var visibleItems = $('#sticky li:visible .feedLink').map(function (i, link) {
        return $(link).parents('li').data('muzli-id');
      }).toArray();

      if (visibleItems && visibleItems.length) {
        $scope.allFeed = data.filter(function(item) {
          return visibleItems.indexOf(item.id) === -1
        });
      } else {
        $scope.allFeed = data;
      }

      //Normalize the top feed click distribution taking [muzli] as a baseline
      // if ($scope.$parent.currentFeedFilter === 'top' && $scope.allFeed[0].source.name === 'muzli') {

      //   var topMuzliItem = $scope.allFeed[0];
      //   var topOtherItem = $scope.allFeed.find(item => item.source.name !== 'muzli');

      //   let muzliClickRatio = topMuzliItem.clicks / topOtherItem.clicks;

      //   $scope.allFeed.sort((a, b) => {

      //     var aClicks = a.source && a.source.name === 'muzli' ? Math.floor(a.clicks / muzliClickRatio) : a.clicks;
      //     var bClicks = a.source && b.source.name === 'muzli' ? Math.floor(b.clicks / muzliClickRatio) : b.clicks;

      //     return bClicks - aClicks;
      //   })

      // }

      //EXPERIMWNTAL
      if ($scope.$parent && $scope.$parent.currentFeedFilter === 'top') {

        $scope.allFeed.sort((a, b) => {

          var aTrajectory = a.clicks - (a.clicksPreviuos || 0);
          var bTrajectory = b.clicks - (b.clicksPreviuos || 0);

          return bTrajectory - aTrajectory;

        })

      }


      $timeout(() => {

        var pinnedItems = $scope.muzliFeed
        .filter(item => {
          return item.isPinned && visibleItems.indexOf(item.id) !== -1;
        }).map(item => item.id)

        let seenPinnedItems = JSON.parse(localStorage.getItem('seenPinnedItems') || '{}');

        pinnedItems.forEach(item => {
          seenPinnedItems[item] = (seenPinnedItems[item] || 0) + 1;
        });

        localStorage.setItem('seenPinnedItems', JSON.stringify(seenPinnedItems));

      });

    });
  }

  function sortByClicked(data, clickedItems) {

    //Filter our already clicked items
    return data.filter(function (item) {
      return clickedItems.indexOf(item.link) === -1;
    })

    //Add already clicked items to the end
    .concat(data.filter(function (item) {
      return clickedItems.indexOf(item.link) > -1;
    }));
  }

  function loadCompleteControl($scope, feedFetcher, clickedItems, muzliData, instagramData, storage) {

    $scope.muzliFeed = sortByClicked(muzliData, clickedItems);

    $scope.dynamicFeed = (instagramData || []).sort((a,b) => {
      return .5 - Math.random();
    });

    //Deffer loading controls to improve rendering
    setTimeout(() => {

      $scope.postClick = function (item, event) {

        var target = $(event.target);

        if (item.video && target.hasClass('postPhoto')) {

          event.preventDefault();
          event.stopPropagation();

          setTimeout(function () {
            target.find('muzli-video > div').click();
          }, 50);

          return;
        }

        if (item.video && (target.hasClass('angular-youtube-wrapper') || target.hasClass('player-image'))) {

          event.preventDefault();
          event.stopPropagation();
          item.playing = true;
          feedFetcher.event.videoClick(item, "sticky");

        } else {
          feedFetcher.event.postClick(item, event, "sticky");
        }

        if (item.source && item.source.name === 'muzli') {
          clickedItems.push(item.link);
          synchMuzliClicked(storage, clickedItems);
        }
      };

      $scope.markHidden = function(item) {

        feedFetcher.event.markHidden(item).then(function(response) {

            var index = $scope.muzliFeed.indexOf(item);

            if (index > -1) {
                $scope.muzliFeed.splice(index, 1);
            }

        });
      }

      $scope.playerVars = feedFetcher.constants.playerVars;
      $scope.openSharer = feedFetcher.event.openSharer;
      $scope.sendSlack = feedFetcher.event.sendSlack;
      $scope.sourceClick = feedFetcher.event.sourceClick;
      $scope.toggleFavorite = feedFetcher.event.toggleFavorite;
      $scope.markNSFW = feedFetcher.event.markNSFW;
      $scope.unmarkNSFW = feedFetcher.event.unmarkNSFW;

    })
  }

  function addShareLinks(feedPage) {

    if (!window.MUZLI_SHARE_SERVER) {
      return;
    }

    if (!JSON.parse(localStorage.enableSharebleLinks || 'false')) {
      return;
    }

    if (window.innerWidth <= 1440) {
      return;
    }

    feedPage.forEach(function(item, index) {

      if (!item.isFrameAllowed) {
        return;
      }

      var listPosition = index;
      var shortUrl = btoa(item.id.slice(0,9))
      var linkOut = window.MUZLI_SHARE_SERVER + shortUrl + '?referrer=muzli&source=muzli&skip=' + listPosition;

      item.link_out = linkOut;
    })
  }

  /*==================================
  =            CONTROLLER            =
  ==================================*/

  allController.$inject = ['$timeout', '$scope', '$q', '$rootScope', '$stateParams', 'sources', 'muzliFeedDefer', 'dynamicFeedDefer', 'feedFetcher', 'storage' , 'trackService'];

  function allController($timeout, $scope, $q, $rootScope, $stateParams, sources, muzliFeedDefer, dynamicFeedDefer, feedFetcher, storage, trackService) {

    console.log('All controller init')

    $rootScope.setFeedFilter = function (filter, $event) {

      if ($event) {
        $event.preventDefault();
        $event.stopPropagation();
      }

      if ($rootScope.currentFeedFilter === filter) {
        return;
      }

      $scope.allFeed = [];
      $rootScope.currentFeedFilter = filter;

      //Send scroll position to top of the tabs if sticky nav is showing
      if ($rootScope.showStickyNav) {

        window.scrollTo({
          top: parseInt($('.lock-height').offset().top - 85),
        });

        $('.tabs.inline').get(0).classList.remove('hidden');
        $('.tabs.sticky').get(0).classList.remove('active');

        $rootScope.showStickyNav = false;

      } else {

        let inlineTab = document.querySelector('.tabs.inline');

        if (window.innerHeight + window.pageYOffset - inlineTab.offsetTop < window.innerHeight/1.5) {

          window.scrollTo({
            top: parseInt($('.lock-height').offset().top - 85),
          });

        }

      }

      fetchAll(feedFetcher, $stateParams.sort, filter)
      .then(function(allFeed) {
        setAllFeed($timeout, $scope, allFeed);
      })

      trackService.track({
        category: 'Feed',
        action: 'Select filter',
        label: filter
      });
    };

    var muzliFeed = muzliFeedDefer.promise;
    var dynamicFeed = dynamicFeedDefer;

    $scope.muzliFeed = [];
    $scope.dynamicFeed = [];
    $scope.sponsored = {};
    $scope.filter = $stateParams.filter;
    $rootScope.currentFeedFilter = $stateParams.filter || 'all';

    //RELOAD
    $rootScope.initScrollTracking($scope,);

    //RELOAD
    feedFetcher.fetchSponsoredPost()
    .then(function (sponsored) {
      sponsored.beacon += '&source=home'
      $scope.sponsored = sponsored;
    });

    var sec;
    setTimeout(function () {
      sec = true;
    }, 500);

    $q.all([
      muzliFeed,
      fetchMuzliClicked(storage),
      dynamicFeed
    ])
    .then(function (res) {

      var fetchedMuzliItems = res[0];
      var clickedItems = res[1];
      var instagramItems = res[2];

      $rootScope.initialLoading = 'loading-muzli-complete';

      loadCompleteControl($scope, feedFetcher, clickedItems, fetchedMuzliItems, instagramItems, storage);

      $timeout(() => {

        //Load all feed
        fetchAll(feedFetcher, $stateParams.sort, $scope.filter)
        .then(async (allFeed) => {

          //Set main feed
          setAllFeed($timeout, $scope, allFeed);


          //Figure out new unread items
          let storageResponse = await storage.get(['lastHomeScrolled'])

          let yesterday = new Date();
          yesterday.setDate(yesterday.getDate() - 1);

          let lastHomeScrolled = (storageResponse && storageResponse.lastHomeScrolled) ? new Date(storageResponse.lastHomeScrolled) : yesterday;

          let unreadSources = allFeed
            .filter(item => {
              return lastHomeScrolled <= new Date(item.created) && item.source;
            })
            .map(item => item.source)

          $rootScope.unreadSources = unreadSources
          .filter((source, index) => {

            // Never show muzli as unread
            if (source.name === 'muzli' || source.name === 'colors') {
                return false;
            }

            return !source.read && unreadSources.indexOf(source) === index;

          })
          .map(source => {

            source.unread = true;

            return source;

          })
          .slice(0, 3);

          //Update unread state on shortcuts
          $rootScope.updateShortcuts();


        })
        .catch(function (err) {

          if (err && err.status === -1 && sec) {
            return;
          }

          $rootScope.setError($scope, 'all-error');

        });

      });

    })
    .catch(function (err) {

      if (err && err.status === -1 && sec) {
        return;
      }

      $rootScope.setError($scope, 'muzli-error');

    });

    $q.all([muzliFeed, $rootScope.scrolledPromise])
    .then(function () {
      $rootScope.initialLoading = 'loading-complete';
    });


    //Sticky navigation handling
    var inlineNavElement;
    var stickyNavElement;

    var navScrollListener = $rootScope.$on('view-scrolled', function(event) {

      if (!inlineNavElement) {
        inlineNavElement = $('.tabs.inline').get(0);
        stickyNavElement = $('.tabs.sticky').get(0);
      }

      if ((window.pageYOffset - 35 > inlineNavElement.offsetTop) && !$rootScope.showStickyNav) {
        stickyNavElement.classList.add('active');
        inlineNavElement.classList.add('hidden');
        $rootScope.showStickyNav = true;
      }

      if ((window.pageYOffset - 35 <= inlineNavElement.offsetTop) && $rootScope.showStickyNav) {
        stickyNavElement.classList.remove('active');
        inlineNavElement.classList.remove('hidden');
        $rootScope.showStickyNav = false;
      }

    });

    $scope.$on("$destroy", function() {
      $rootScope.showStickyNav = false;
      navScrollListener();
    })

  }


  /*==============================
  =            CONFIG            =
  ==============================*/


  config.$inject = ['$provide', '$stateProvider', '$urlRouterProvider', '$compileProvider', '$sceDelegateProvider','$animateProvider'];

  function config($provide, $stateProvider, $urlRouterProvider, $compileProvider, $sceDelegateProvider, $animateProvider) {

    $compileProvider.debugInfoEnabled(false);

    $compileProvider.commentDirectivesEnabled(false);
    $compileProvider.cssClassDirectivesEnabled(false);
    $compileProvider.imgSrcSanitizationWhitelist(/^\s*(https?|ftp|mailto|chrome-extension|safari-extension):/);
    $sceDelegateProvider.resourceUrlWhitelist(['self', 'safari-extension://**']);

    $animateProvider.classNameFilter(/angular-animate/);

    $provide.decorator("$exceptionHandler", ["$delegate", "trackService", function ($delegate, trackService) {
      return function (exception, cause) {
        
        var shouldDelegate = trackService.trackError(exception);

        // (Optional) Pass the error through to the delegate formats it for the console
        if (shouldDelegate) {
          $delegate(exception, cause);
        }
      };
    }]);

    $urlRouterProvider.otherwise(function ($injector) {

      var $state = $injector.get('$state');

      if (window.LITE_CHECK) {
        $state.go('lite-check');
        return;
      }

      $state.go('all');

    });

    /*==============================
    =            Routes            =
    ==============================*/

    $stateProvider.state('all', {
      templateUrl: 'modules/muzli/home.html',
      controller: allController,
      reloadOnSearch: false,
      params: {
        sort: 'created',
        filter: undefined,
      },
      resolve: {

        dynamicFeedDefer: ['feedFetcher', 'userService', '$rootScope', async function (feedFetcher, userService, $rootScope) {

          let user = await userService.fetch();

          return fetchDynamicFeed(feedFetcher, $rootScope)
          .catch(function(err) {
            console.error('Failed to fetch Muzli Instagram feed')
          })

        }],

        muzliFeedDefer: ['$q', 'feedFetcher', '$stateParams', 'userService',
          function ($q, feedFetcher, $stateParams, userService) {

            let defer = $q.defer();
            let sort = $stateParams.sort;

            fetchMuzli(feedFetcher, sort, userService).then(defer.resolve, defer.reject);

            return defer;

        }]

      }
    });

  }


  /*===========================
  =            RUN            =
  ===========================*/

  run.$inject = ['$timeout', '$rootScope', '$q', '$state', '$interval', 'feedFetcher', 'userService', 'socialService', 'trackService', 'storage', 'sources_list', 'sources', 'experiments', 'chrome'];

  function run($timeout, $rootScope, $q, $state, $interval, feedFetcher, userService, socialService, trackService, storage, sources_list, sources, experiments, chromeService,) {
    
    var revealDistance = 0;
    var resetRevealTimeout;
    var pauseScrollTimeout;
    var scrollDefer;
    var nextState;

    //Base for A/B testing using GA custom dimensions
    // experiments.getExperiment('50/50').then(function(testExperiment) {
    //   if (testExperiment.variant === 1) {
    //       trackService.setDimension('dimension2', 'First-half');
    //   } else {
    //       trackService.setDimension('dimension2', 'Second-half');
    //   }
    // });

    trackService.onLoad(storage, sources_list);

    $rootScope.$state = $state;
    $rootScope.contactLinks = contactLinks;
    $rootScope.shortcutsActive = JSON.parse(localStorage.shortcutsActive || 'false');

    window.muzli.closeOnEsc.push(function () {
      $rootScope.menuOpen = false;
    });

    $rootScope.updateShortcuts = function() {
      sources.getShortcuts().then(function(shortcuts) {

        $rootScope.shortcuts = shortcuts;

        $timeout(() => {
          $rootScope.showShortcuts = true;
        })

      });
    }

    $rootScope.$watchCollection('user.shortcuts', $rootScope.updateShortcuts);

    function moveToFullView () {

      $rootScope.isSwitchedToHalfView = true;
      $rootScope.feedVisibleClass = 'full-view';

      $('main, #overlay').off("wheel scroll")

    }

    function setView() {

      storage.get(['lite', 'liteOverride', 'halfView']).then(function (res) {

        var isSwitchedToHalfView = angular.isDefined(res.halfView) ? !!res.halfView : !!res.lite;

        if (window.muzli.getRuntime().id !== "non_chrome") {
          $rootScope.isLiteVersion = res.lite;
          $rootScope.isMuzliHomepage = res.liteOverride;
        }

        //Remove last week feed instantly if full-view is set
        if (isSwitchedToHalfView) {
          localStorage.removeItem('fetchLastWeek');
        }

        $rootScope.isSwitchedToHalfViewIndicator = isSwitchedToHalfView;
        $rootScope.isSwitchedToHalfView = isSwitchedToHalfView;

        $rootScope.feedVisibleClass = isSwitchedToHalfView ? 'full-view' : '';

      });
    }

    function init() {

      scrollDefer = $q.defer();

      $rootScope.initialLoading = 'loading-start';

      $rootScope.scrolledPromise = scrollDefer.promise;

      userService
      .getUserBundle()
      .then(function(bundleName) {

        let tabs = [
          {
            name: 'design',
            title: 'Design',
          }, {
            name: 'tech',
            title: 'Tech',
          }, {
            name: 'culture',
            title: 'Culture',
          }, {
            name: 'news',
            title: 'News',
          },
        ]

        $rootScope.focusedTab = tabs.find(tab => tab.name === bundleName);
        $rootScope.navTabs = tabs.filter(tab => tab !== $rootScope.focusedTab);

      })

      //Track when user switches to full view
      $rootScope.scrolledPromise.then(function(label) {

        localStorage.removeItem('fetchLastWeek');

        $timeout(function() {
          $('main img.wait-for-canvas').trigger('loadCanvas');
        });

        storage.set({
          lastHomeScrolled: new Date(),
        })

        trackService.track({
          category: 'Home',
          action: 'Scroll',
          label: label,
        });

      })

      setView();

      storage.get(['installDate', 'updateDate']).then(function (res) {

        //Sume ugly hack to parse installDate
        function sliceDate(dateString) {

            var slicedDate = dateString.slice(0, 2) + ' ';
            slicedDate += dateString.slice(2, 4) + ' ';
            slicedDate += dateString.slice(4, 8);

            return new Date(slicedDate);
        };

        $rootScope.bootstrapped = true;
        $rootScope.installDate = sliceDate(res.installDate || '01012020');
        $rootScope.updateDate = sliceDate(res.updateDate || '01012020');

        $(window).on('beforeunload', function () {
          window.muzli.pageChange();
        });

        //If home screen is not full feed - hook scroll events to reveal it
        if (!$rootScope.isSwitchedToHalfView) {

          //Since header overlays home screen, set different event listener for it 
          $timeout(() => {
            $('header').on("wheel scroll", e => {
              moveToFullView();
              $('header').off("wheel scroll");
            });
          }, 1000);

          $('main, #overlay').on("wheel scroll", e =>  {

            $rootScope.feedVisibleClass = 'full-view';
            $rootScope.initialLoading = 'loading-scrolled';
            $rootScope.$broadcast('user-scrolled');
            scrollDefer.resolve('User scrolled');

            //Bust FTX state
            let index = $rootScope.vm.ftxLeft.indexOf('scroll');

            if (index !== -1) {

              $rootScope.vm.ftxLeft.splice(index, 1);

              storage.set({
                  ftxLeft: $rootScope.vm.ftxLeft,
              })

              $rootScope.vm.showFtx = false;
            }

          });
        }

      });
    }

    init();

    $rootScope.$on('loadLanding', init);
    $rootScope.$on('muzliSetView', setView);
    $rootScope.$on('muzliMoveToFullView', moveToFullView);

    $rootScope.$on('$stateChangeStart', function (event, state) {

      nextState = state.name;
      $rootScope.errors = [];

      window.muzli.removeTooltips('#feed [title]');
    });

    $rootScope.$on('$stateChangeSuccess', function (event, state, toParams) {

      $rootScope.currentFeedSort = toParams.sort || 'created';

      console.log('State changed to: ' + state.name);
    });

    $rootScope.vm = {
      ftxLeft: [],
    };

    $rootScope.events = {
      sidebar: {
        show: function () {
          trackService.track({
            category: 'Sidebar',
            action: 'Show',
            label: 'sidebar'
          });
        },
        clickLink: function (name, event) {
          event.stopPropagation();

          trackService.track({
            category: 'Sidebar',
            action: 'Click',
            label: 'Site Link: ' + name
          });
        }
      },
      settingsMenu: {
        clickLink: function (text) {
          trackService.track({
            category: 'Settings menu',
            action: 'Click',
            label: text
          });
        }
      },
      quickAccess: {
        click: function (url) {
          trackService.track({
            category: 'Quick access',
            action: 'Click',
            label: url
          });
        }
      }
    };

    $state.goHome = function () {

      $state.go('all');

      window.scrollTo({
        top: 0,
        behavior: 'smooth',
      })

      let inlineTabs = $('.tabs.inline').get(0);
      let stickyTabs = $('.tabs.sticky').get(0);

      if (inlineTabs) {
        inlineTabs.classList.remove('hidden');
      }

      if (stickyTabs) {
        stickyTabs.classList.remove('active')
      }
      
      $rootScope.showStickyNav = false;

      let shortcutContainer = document.querySelector('.shortcuts ul');

      if (shortcutContainer) {
        shortcutContainer.scrollTo({
          top: 0,
          behavior: 'smooth',
        })
      }

    };

    $rootScope.reloadPage = function () {
      window.location.reload();
    };

    $rootScope.shakeBlocker = function(event) {

      var shakeElement = $('.blocker .wrapper');

      $(shakeElement).addClass('shake');

      setTimeout(function() {
        $(shakeElement).removeClass('shake');
      }, 750);
    }

    $rootScope.toggleShortcuts = function() {

      $rootScope.shortcutsActive = !$rootScope.shortcutsActive;
      localStorage.shortcutsActive = $rootScope.shortcutsActive;

      trackService.track({
        category: 'Shortcuts',
        action: 'Click Pull-tab',
        label: $rootScope.shortcutsActive ? 'open' : 'close',
      });

      //FTX state bust
      if ($rootScope.vm.ftxLeft.indexOf('shortcuts') !== -1) {
          $rootScope.showFtx('shortcuts')
      }

      if ($rootScope.vm.showFtx === 'shortcuts') {
          $rootScope.vm.showFtx = false;
      }
    }

    $rootScope.setTheme = function (name) {
      $rootScope.theme = name;
      userService.setData({
        theme: name
      });
    };

    $rootScope.activeBundle = 'design';

    $rootScope.selectBundle = function(bundleName, reload, trackFtx) {

        $rootScope.activeBundle = bundleName;

        userService.setData({
            selectedBundle: bundleName
        }).then(() => {

          if (trackFtx) {
            trackService.trackPageView('/ftx/v3/welcome/bundle/' + bundleName, 'Welcome Focus');
          }

          if (reload) {
            location.reload();
          }
        })
    }

    $rootScope.toggleSharebleLinks = function (value) {

      $rootScope.enableSharebleLinks = value;
      localStorage.enableSharebleLinks = value;

      userService.setData({
        enableSharebleLinks: value
      });
      
    };

    $rootScope.togglePalettes = function (value) {

      $rootScope.enablePalettes = value;
      localStorage.enablePalettes = value;

      userService.setData({
        enablePalettes: value
      });
    };

    $rootScope.closeAlert = function (alert) {
      userService.markReadAlert({
        id: alert.id
      }).then(function(response) {

        $rootScope.user.unread_alerts = response;

        var index = $rootScope.alerts.indexOf(alert);
        $rootScope.alerts.splice(index, 1);

        trackService.track({
          category: 'Bar notifications',
          action: 'Dismiss click',
          label: $('<span>' + alert.content + '</span>').text(),
        });

      });
    };

    $rootScope.logAlertCta = function(alert, $event) {

      if (!$($event.target).is('a')) {
        return;
      }

      trackService.track({
        category: 'Bar notifications',
        action: 'CTA click',
        label: $($event.target).text() + ' | ' + $('<span>' + alert.content + '</span>').text(),
      });
    };

    $rootScope.getBodyClass = function () {

      var name = 'feed-';
      var currentSource = $rootScope.currentSource;

      if (currentSource && currentSource.name) {
        name += currentSource.name;
      }
      else {
        name += currentSource
      }

      if ($rootScope.feedVisibleClass) {
        name += (' ' + $rootScope.feedVisibleClass);
      }

      if ($rootScope.initialLoading) {
        name += (' ' + $rootScope.initialLoading);
      }

      return name;
    };

    $rootScope.clickLogo = function () {

      trackService.track({
        category: 'Sidebar',
        action: 'Click',
        label: 'muzli logo'
      });

      $state.goHome();
    };

    $rootScope.onSearch = function (event, value) {
      if (event.which == 13) {
        document.location = "http://muz.li/search/?q=" + value;
      }
    };

    $rootScope.showFtx = function (name) {

      $timeout(function () {
        $rootScope.vm.showFtx = name;
      }, 350);

      let index = $rootScope.vm.ftxLeft.indexOf(name);
      $rootScope.vm.ftxLeft.splice(index, 1);

      storage.set({
          ftxLeft: $rootScope.vm.ftxLeft,
      })
    }

    $rootScope.openMenu = function () {

      $rootScope.menuOpen = true;

      //FTX state bust
      if ($rootScope.vm.ftxLeft.indexOf('settings') !== -1) {
        $rootScope.showFtx('settings')
      }

      trackService.track({
        category: 'Settings menu',
        action: 'Click',
        label: 'Open'
      });

      if ($rootScope.refreshCookieData) {
        $rootScope.refreshCookieData();
      }

      //???
      $(document).on("click.menuOpen", function (e) {
        if ($(e.target).parents('.feedLink').length) {
          e.preventDefault();
        }
      });
    };

    $rootScope.closeMenu = function () {

      $rootScope.menuOpen = false;
      $rootScope.uninstall = false;

      $("aside").removeClass("uninstall");
      $(document).off("click.menuOpen");

      if ($rootScope.vm.showFtx === 'settings') {
        $rootScope.vm.showFtx = false;
      }
    };

    $rootScope.bodyKeyUp = function (event) {
      if (event.keyCode === 27) {

        $("aside").removeClass("uninstall");

        window.muzli.closeOnEsc.forEach(function (item) {
          item();
        });
      }
    };

    $rootScope.bodyKeyDown = function (event) {

      if (event.metaKey || event.ctrlKey || event.shiftKey || event.altKey) {
        return;
      }

      var c = String.fromCharCode(event.keyCode);
      var isWordCharacter = c.match(/\w/);
      var isBackspaceOrDelete = (event.keyCode == 8 || event.keyCode == 46);
      var noFocus = $("input[type=text],input[type=url],input[type=email]").is(':focus');

      //Open sidebar and start typing into search
      if (((isWordCharacter || isBackspaceOrDelete) && !noFocus)) {

        if ($rootScope.showSidebar) {
          $('input[ng-model=searchSources]').focus() 
        } else {
          $('input[name="qMuzli"]').focus();
        }

      }

      //Close sidebar on ESC
      if (event.keyCode == 27) {

        $("input[ng-model=searchSources]").blur();
        $rootScope.showSidebar = false;
        $rootScope.searchSources = '';

        if ($state.current.name === 'feed') {
          $state.go('all')
        }

      }


    };

    $rootScope.clickOutsideSidebar = function () {
      $("input[ng-model=searchSources]").blur();
      $rootScope.showSidebar = false;
    };

    $rootScope.twitterFollow = function () {
      window.open('https://twitter.com/intent/follow?screen_name=usemuzli', 'follow', 'height=400,width=550');
    };

    $rootScope.reload = function () {
      if (!$rootScope.user) {
        window.location.reload();
        return;
      }
      $state.reload();
    };

    $rootScope.sortFeed = function (sort) {

      var current = $state.current.name;
      var params = { sort: sort };

      $state.go(current, params, {reload: true});

      var sortValue = (sort === 'virality') ? 'Popular' : 'Recent';

      trackService.track({
        category: 'Feed',
        action: 'Sort',
        label: sortValue
      });
    };

    $rootScope.setError = function ($scope, error) {
      $timeout(function () {
        if (!$scope.$$destroyed && !$rootScope.userNavigatingAway) {
          $rootScope.errors.push(error);
        }
      }, 100);
    };

    $rootScope.initScrollTracking = function($scope) {

      var scrollDistance = 0;
      var currentState = $scope.$state.current.name;
      var currentSource = $scope.$state.params.name;

      function trackScrollDistance(e) {
        $rootScope.$broadcast('view-scrolled');
        scrollDistance -= e.originalEvent.wheelDeltaY;
      }

      $('body').on("wheel scroll", trackScrollDistance);

      $scope.$on("$destroy", function() {

        chromeService.sendMessage({
          scrollDistance: scrollDistance,
          state: currentState,
          source: currentSource,
        })

        $('body').off("wheel scroll", trackScrollDistance);

      });

      window.addEventListener("beforeunload", function(e){

        chromeService.sendMessage({
          scrollDistance: scrollDistance,
          state: currentState,
          source: currentSource,
        })

      }, false);
    }

    $rootScope.closeVideoPopup = function () {

      $rootScope.currentPlayingItem = false;

      $('.video-container iframe, .video-container video').remove();
      $('.video-container').removeClass('pop');
      $('.video-container').css({
        width: '',
        height: '',
      })
    }


    if (window.chrome || window.isMuzliSafari) {

      $rootScope.jumpToRecent = function (skipScroll) {

        //Bust FTX state
        let index = $rootScope.vm.ftxLeft.indexOf('scroll');

        if (index !== -1) {
          $rootScope.vm.ftxLeft.splice(index, 1);

          storage.set({
              ftxLeft: $rootScope.vm.ftxLeft,
          })

          $rootScope.vm.showFtx = false;
        }

        moveToFullView();

        $rootScope.feedVisibleClass = 'full-view';
        $rootScope.initialLoading = 'loading-scrolled';
        $rootScope.$broadcast('user-scrolled');

        scrollDefer.resolve('User clicked scroll trigger');

        if (!skipScroll) {
          window.scrollTo({
            top: parseInt(window.innerHeight / 2),
            behavior: 'smooth',
          });
        }

        window.blockScrollEvents = true;

        setTimeout(() => {
          window.blockScrollEvents = false;
        }, 500)

      };

      $rootScope.toggleMinimalView = function () {

        $rootScope.isSwitchedToHalfViewIndicator = !$rootScope.isSwitchedToHalfViewIndicator

        var setData = userService.setData({
          halfView: $rootScope.isSwitchedToHalfViewIndicator
        }, !$rootScope.isSwitchedToHalfViewIndicator);

        if ($rootScope.isSwitchedToHalfViewIndicator) {
          moveToFullView();
        } else {
          setData.then(function() {
            location.reload();
          });
        }
      };
    }


    //When user returns to tab, refresh cookie data
    document.addEventListener("visibilitychange", function() {
      if (!document.hidden && $rootScope.refreshCookieData) {
        $rootScope.refreshCookieData();
      }
    });

    //Set flag if user is navigating away from tab
    window.addEventListener("beforeunload", function(e){
      $rootScope.userNavigatingAway = true;
    }, false);

  }

  try {
    var moduleTemplate = angular.module('muzli-template')
  }
  catch (e) {
    moduleTemplate = angular.module('muzli-template', [])
  }

  angular.module('muzli', [
    moduleTemplate.name,
    'ngAnimate',
    'ui.router',
    'angular-click-outside',
    'bootstrap',
    'user',
    'search',
    'sources',
    'feed',
    'chrome',
    'community',
  ])
  .constant('R', window.R)
  .value('server', function() {
    return window.MUZLI_SERVER
  })
  .config(config)
  .run(run);

})();
