(function () {

    var details = window.muzli.getDetails();

    trackService.$inject = ['$window'];

    function trackService($window) {

        let guidPromise;

        return {
            getGuid: getGuid,
            onLoad: onLoad,
            track: track,
            trackPageView: trackPageView,
            trackError: trackError,
            setDimension: setDimension,
        };

        function getGuid(storage) {
            guidPromise = guidPromise || storage.get("UUID").then(function (obj) {
                var uuid = obj.UUID;
                if (!uuid) {
                    uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
                        var r = crypto.getRandomValues(new Uint8Array(1))[0] % 16 | 0,
                            v = c == 'x' ? r : (r & 0x3 | 0x8);
                        return v.toString(16);
                    });

                    storage.set({
                        'UUID': uuid
                    });
                }
                return uuid;
            });

            return guidPromise;
        }

        function onLoad(storage, sources_list) {

            setTimeout(() => {

                $window.intercomSettings = {
                    app_id: "oy912gcy",
                };

                window.activateIntercom = _setupIntercom; 
                
            }, 0)

            getGuid(storage).then(function (uuid) {
                _sendGAOnLoad({
                    label: 'User ID',
                    value: uuid
                });
            });

            if (!$window.localStorage.getItem('sentSideBarTacking')) {
                $window.localStorage.setItem('sentSideBarTacking', "yes");
                sources_list.forEach(function (source) {
                    _sendGA({
                        category: 'Sidebar',
                        action: 'Change',
                        label: 'Source: ' + source.name,
                        value: 1
                    });
                });
            }
        }

        function track(config) {
            _sendGA(config);
        }

        function trackPageView(url, title, callback) {

            if (!window.ga) {
                return;
            }

            window.ga('send', {
                'hitType': 'pageview',
                'page': url,
                'title': title,
                'hitCallback': callback || function () {},
            });

        };

        function setDimension(dimension, value) {

            if (dimension === 'dimension1') {
                console.error('Error: dimension1 is reserved for version tracking only');
                return;
            }

            try {

                window.ga('set', dimension, value);

            } catch (err) {
                console.error("Google analytics error", err);
            }
        }

        function trackError(exception, log) {
            if (log) {
                console.error(exception);
            }

            return true;
        }

        function _sendGA(config) {

            var category = config.category;
            var action = config.action;
            var label = config.label;
            var value = config.value;
            var fields = config.fields;

            ga('send', 'event', category, action, label, value, fields);

        }

        function _sendGAOnLoad(config) {
            config.category = 'App';
            config.action = 'Load';
            _sendGA(config);
        }

        function _setupIntercom() {

            var w = $window;
            var ic = w.Intercom;

            if (typeof ic === "function") {
                ic('reattach_activator');
                ic('update', w.intercomSettings);
            } else {
                var d = document;
                var i = function () {
                    i.c(arguments);
                };
                i.q = [];
                i.c = function (args) {
                    i.q.push(args);
                };
                w.Intercom = i;
                var l = function () {
                    var s = d.createElement('script');
                    s.type = 'text/javascript';
                    s.async = true;
                    s.src = 'lib/intercom.js';
                    var x = d.getElementsByTagName('script')[0];
                    x.parentNode.insertBefore(s, x);
                };
                l();
            }

        }

    }

    angular.module('muzli').factory('trackService', trackService);

})();