(function () {

  service.$inject = ['$q', '$http', '$timeout', '$rootScope', 'sources_list', 'storage', 'userService', 'bundles_list'];
  function service ($q, $http, $timeout, $rootScope, sourcesList, storage, userService, bundles_list) {

    function findByName (sources, name) {
      return sources.filter(function (item) {
        return item.name === name;
      })[0];
    }

    return {

      //Source demotion
      getDemotedSources() {

        return userService
          .fetch()
          .then(function(user) {
            return user.demotedSources || [];
          })
      },

      //Shortcuts
      demoteSource(source) {

        var demotedSources = $rootScope.user.demotedSources || [];
        var index = demotedSources.indexOf(source.name);

        if (index === -1) {
          demotedSources.push(source.name)
        }

        userService.setData({
          demotedSources: demotedSources,
        });

        source.isDemoted = true;
      },

      reactivateSource(source) {

        var demotedSources = $rootScope.user.demotedSources || [];
        var index = demotedSources.indexOf(source.name);

        if (index !== -1) {

          demotedSources.splice(index, 1);

          userService.setData({
            demotedSources: demotedSources,
          });
        }

        source.isDemoted = false;
      },
      
      getShortcuts() {

        //Add singleton 
        
        return userService
          .fetch()
          .then(async (user) => {

            let _shortcuts = user.shortcuts;

            //If no shortcuts array is defined, treat is as FTX
            if (!_shortcuts) {

              //Resolve get right bundle
              let bundleName = await userService.getUserBundle();

              let initialShortcuts = {
                'design': [
                  'muzli',
                  'invisionapp',
                  'muzli_blog',
                  'behance',
                  'dribbble',
                  'designer_news',
                  'awwwards',
                  'sidebar',
                ],
                'tech': [
                  'producthunt',
                  'techcrunch',
                  'the_verge',
                  'smashing_mag',
                  'engadget',
                  'codepen',
                  'hacker_news',
                  'mashable',
                ],
                'culture': [
                  'mashable',
                  'ted',
                  'colossal',
                  'vox',
                  'vice',
                  'my_modern_met',
                  'daily_dot',
                  'buzzfeed',
                ],
                'news': [
                  'cnn',
                  'forbes',
                  'vb',
                  'nytimes',
                  'npr',
                  'vice',
                  'futurism',
                ],
              }

              _shortcuts = [];

              //Find out sources that user explicitly added
              user.sources
              .filter(source => source.enabled)
              .forEach(source => {
                _shortcuts.push(source.name)
              })

              //If no sources were disabled, 
              if (!_shortcuts.length) {
                _shortcuts = initialShortcuts[bundleName] || initialShortcuts['design'];
              }

              user.shortcuts = _shortcuts;

            }

            return _shortcuts
              .map(sourceName => {

                let source = findByName(sourcesList, sourceName);

                if (!source) {
                  return false;
                }

                return { 
                  name: sourceName,
                  title: source.title,
                  unread: source.unread,
                }

              })
              .filter(source => !!source)

          })
      },

      clearShortcuts() {

        $rootScope.clearedShortcuts = $rootScope.user.shortcuts;
        $rootScope.user.shortcuts = [];

        userService.setData({
          shortcuts: [],
        });
      },

      restoreShortcuts() {

        $rootScope.user.shortcuts = $rootScope.clearedShortcuts;
        $rootScope.clearedShortcuts = [];

        userService.setData({
          shortcuts: $rootScope.user.shortcuts,
        });
      },

      addShortcut(source) {

        var _shortcuts = $rootScope.user.shortcuts;
        var index = _shortcuts.indexOf(source.name);

        if (index === -1) {
          _shortcuts.push(source.name)
        }

        userService.setData({
          shortcuts: _shortcuts,
        });

        var staticSource = sourcesList.find(staticSource => {
          return staticSource.name === source.name
        })

        if (staticSource) {
          staticSource.isShortcut = true;
        }

      },

      removeShortcut(source) {

        var _shortcuts = $rootScope.user.shortcuts;
        var index = _shortcuts.indexOf(source.name);

        if (index !== -1) {
          _shortcuts.splice(index, 1);
        }

        userService.setData({
          shortcuts: _shortcuts,
        });

        var staticSource = sourcesList.find(staticSource => {
          return staticSource.name === source.name
        })

        if (staticSource) {
          staticSource.isShortcut = false;
        }

      },


      //Legacy source functionality
      findByName: findByName.bind(this, sourcesList),

      fetch: function () {

        return this
        .getShortcuts()
        .then(shortcuts => {

          shortcuts.forEach(shortcut => {

            var source = sourcesList.find(source => {
              return shortcut.name === source.name
            })

            if (source) {
              source.isShortcut = true;
            }

          })

          return sourcesList;

        })

      },

    }
  }

  angular.module('sources')
    .factory('sources', service);

})();
