angular.module('sources').controller('welcomeController', ['$scope', '$state', 'storage', 'trackService', 'userService', '$rootScope',
    function ($scope, $state, storage, trackService, userService, $rootScope) {

        console.log('Welcome controller init')

        var isWindows = navigator.appVersion.indexOf('Win') !== -1;
        var areFacesLifted = false;
        var welcomeContainer = $('body .welcome');

        var face1Element = $('.face-1');
        var face2Element = $('.face-2');
        var face3Element = $('.face-3');

        document.body.classList.add('lock-scroll')

        //Workaround for Windows scroll-snap bug
        if (isWindows) {
            welcomeContainer[0].style['scroll-snap-type'] = 'initial'; 
        }

        //Fetch user to know initial parameters, that can vary by experiment
        userService.fetch().then(user => {

            $scope.user = user;

            storage.set({
                allowAnonymous: user.allowAnonymous,
            })

            if (user.allowAnonymous !== undefined) {
                trackService.setDimension('dimension7', user.allowAnonymous ? 'anonymous' : 'non-anonymous');
            }

        })

        function moveFaces(scrollProgress) {

            var left = 360 + (240 / 100 * scrollProgress);
            var rotate = 11 + (10 / 100 * scrollProgress);
            var viewportCompensation = (window.innerHeight - 650) / 125;
            var top = scrollProgress / (2 + viewportCompensation);

            face1Element.css({
                top: 'calc(540px - ' + top + 'vh)',
                left: 'calc(50% - ' + left + 'px)',
                transform: 'rotate(-' + rotate + 'deg)',
            })

            face2Element.css({
                top: 'calc(500px - ' + scrollProgress * 1.45 + 'vh)',
            })

            face3Element.css({
                top: 'calc(540px - ' + top * 1.2 + 'vh)',
                right: 'calc(50% - ' + left + 'px)',
                transform: 'rotate(' + rotate + 'deg)',
            })

        }

        welcomeContainer.on("wheel scroll", function (event) {
            
            var scrollProgress = (welcomeContainer.scrollTop() / window.innerHeight) * 100;

            if (scrollProgress <= 100) {

                moveFaces(scrollProgress);

                areFacesLifted = true;

            }
        });

        $scope.nextFrame = function(type) {

            trackService.trackPageView('/ftx/v3/welcome/' + type, 'Welcome Step');

            welcomeContainer[0].scrollTo({
              top: welcomeContainer.scrollTop() + window.innerHeight,
              left: 0,
              behavior: 'smooth'
            });

            //Failsave is scroll event didn't fire
            setTimeout(() => {

                if (document.querySelector('.welcome').scrollTop >= window.innerHeight && !areFacesLifted) {
                    moveFaces(100);
                }

            }, 3000);
            
        }

        $scope.v3SignIn = async function(signInFunction) {

            //Experimental one-off state trigger
            await userService.getUserBundle().then(bundleName => {
                if (bundleName === 'design') {
                    localStorage.fetchLastWeek = true;
                }
            })

            storage.set({
                showedBundles: true,
            })
            .then(signInFunction)
        }

        $scope.finishWelcome = async function() {

            //Experimental one-off state trigger
            await userService.getUserBundle().then(bundleName => {
                if (bundleName === 'design') {
                    localStorage.fetchLastWeek = true;
                }
            })

            trackService.trackPageView('/ftx/v3/welcome/finish', 'Welcome Finish');
            
            $state.go('all');

            storage.set({
                showedBundles: true,
            })
        }

        $scope.skipSignIn = async function() {

            //Experimental one-off state trigger
            await userService.getUserBundle().then(bundleName => {
                if (bundleName === 'design') {
                    localStorage.fetchLastWeek = true;
                }
            })

            trackService.trackPageView('/ftx/v3/welcome/skip', 'Welcome Skip Sign in');

            if (window.isMuzliSafari) {
                $rootScope.vm.ftxLeft = ['settings', 'scroll', 'shortcuts', 'sidebar'];
            } else {
                $rootScope.vm.ftxLeft = ['settings', 'scroll', 'most-visited', 'shortcuts', 'sidebar'];
            }

            storage.set({
                ftxLeft: $rootScope.vm.ftxLeft,
                showedBundles: true,
            })

            $rootScope.vm.showFtx = 'scroll';

            $state.go('all');
        }

        $scope.$on("$destroy", function() {
            welcomeContainer.off("wheel scroll");
            document.body.classList.remove('lock-scroll');
        });

    }
]);
