(function() {

    favoritesController.$inject = ['$scope', '$rootScope', '$timeout', 'feedFetcher', 'userService', 'trackService'];

    function favoritesController($scope, $rootScope, $timeout, feedFetcher, userService, trackService) {

        window.muzli.pageChange();
        $rootScope.initScrollTracking($scope);
        $rootScope.areHomeImagesLoaded = true;

        trackService.track({
            category: 'Sidebar',
            action: 'Click',
            label: 'Source: favorites'
        });

        $rootScope.feedVisibleClass = 'full-view';
        $rootScope.initialLoading = 'loading-complete';

        feedFetcher
        .fetch('favorites', 300)
        .then(function(res) {
            
            $scope.items = res.data;

        }).catch(function(error) {

            console.error(error);

            $rootScope.setError($scope, 'error');

        });

        $rootScope.clearAllFavorites = function() {
            userService.clearFavorites();
            $scope.items = [];
            $rootScope.user.favoriteCount = 0;
        }
    }

    config.$inject = ['$httpProvider', '$stateProvider'];

    function config($httpProvider, $stateProvider) {

        $stateProvider.state('favorites', {
            templateUrl: 'modules/user/favorites.html',
            controller: favoritesController
        });

        $httpProvider.interceptors.push(['$q', 'server', '$rootScope', '$injector', 'storage',
          function($q, server, $rootScope, $injector, storage) {

            var token = storage.getSync('token');

            return {

                request: function(config) {

                    //Force V2 for color feed
                    if (config.url.indexOf('/feed/colors') !== -1) {
                        config.url = config.url.replace(window.MUZLI_SERVER, window.MUZLI_SERVER_V2)
                    }

                    if (config.skipAuth || !token || config.url.indexOf(server()) !== 0) {
                        return config;
                    }

                    if (config.method === "GET") {
                        config.params = config.params || {};
                        config.params.Authorization = 'Bearer ' + token;
                    } else {
                        config.headers.Authorization = 'Bearer ' + token;
                    }

                    return config;
                },

                responseError: function(rejection) {

                    if (rejection.status === 404) {
                        return $q.reject(rejection);
                    }

                    //Falback to V1 API if it fails
                    if (rejection.config.url.indexOf(window.MUZLI_SERVER_V2) === 0 && window.MUZLI_SERVER_V1 && rejection.status !== -1 && !rejection.config.isFallback) {

                        console.warn('V2 API request failed. Falling back to V1.')
                        window.MUZLI_SERVER = window.MUZLI_SERVER_V1

                        rejection.config.url = rejection.config.url.replace(window.MUZLI_SERVER_V2, window.MUZLI_SERVER)
                        rejection.config.isFallback = true;

                        var $http = $injector.get('$http');
                        return $http(rejection.config);
                    }
                    
                    if (rejection.status === 401) {
                        $rootScope.$broadcast('http:401');
                    }

                    return $q.reject(rejection);
                }
            };
        }]);
    }

    run.$inject = ['$state', 'trackService', 'userService', '$rootScope', '$timeout', '$q', 'storage', 'socialService'];

    function run($state, trackService, userService, $rootScope, $timeout, $q, storage, socialService) {

        $rootScope.resolveUser = $q.defer();
        $rootScope.vm = $rootScope.vm || {};

        /*=============================================
        =            Resolve user from API            =
        =============================================*/

        //Set user from local storage to render UI more fluently
        try {
            $rootScope.user = JSON.parse(localStorage.getItem('user'));
        } catch (e) {
            $rootScope.user = {}
        }

        //Set Different API endpont if user is in V2 bucket
        if ($rootScope.user && $rootScope.user.apiVersion === 'v2') {
            
            window.MUZLI_SERVER_V1 = window.MUZLI_SERVER;
            window.MUZLI_SERVER = window.MUZLI_SERVER_V2 || window.MUZLI_SERVER_V1;

            window.MUZLI_AD_SERVER = window.MUZLI_AD_SERVER_V2 || window.MUZLI_AD_SERVER;
            window.MUZLI_SHARE_SERVER = window.MUZLI_SHARE_SERVER_V2 || window.MUZLI_SHARE_SERVER;

            if (window.resolveAdminServer) {
                window.resolveAdminServer('v2')
            }
            
        } else {
            if (window.resolveAdminServer) {
                window.resolveAdminServer('v1')
            }
        }

        userService.fetch().then(function(user) {

            $rootScope.user = user;
            localStorage.setItem('user', JSON.stringify($rootScope.user));

            $rootScope.resolveUser.resolve(user);
            $rootScope.isUserResolved = true;

            if (user.id && !user.anonymous) {

                //Init state after user signs up
                switch (window.REGISTERED) {

                    case 'ftx':

                        if (window.isMuzliSafari) {
                            $rootScope.vm.ftxLeft = ['settings', 'scroll', 'shortcuts', 'sidebar'];
                        } else {
                            $rootScope.vm.ftxLeft = ['settings', 'scroll', 'most-visited', 'shortcuts', 'sidebar'];
                        }

                        storage.set({
                            ftxLeft: $rootScope.vm.ftxLeft,
                            showedBundles: true,
                        })

                        $rootScope.vm.showFtx = 'scroll';

                        break;

                    case 'favorite':

                        $state.go('favorites');

                    //Default tab open w/o redirect from server
                    case undefined:

                        storage.get(['ftxLeft', 'showedBundles']).then(function (res) {
                            
                            //Set FTX dialogs still left unseen
                            $rootScope.vm.ftxLeft = res.ftxLeft || [];

                            //Set FTX reminder for Most-visitet if it's not seen yet
                            var tomorrow = new Date($rootScope.installDate);
                            tomorrow.setDate(tomorrow.getDate() + 1);

                            if (tomorrow < new Date() && $rootScope.vm.ftxLeft.indexOf('most-visited') !== -1) {                         
                                $rootScope.showFtx('most-visited')
                            }

                        })

                        break;

                    default:

                        storage.set({
                            lastLoginProvider: user.provider.toLowerCase(),
                            lastLogin: new Date().toString(),
                        })

                        break;
                }

                //If user didn't provided email, show error
                if (!window.REGISTERED && user.provider && user.provider.toLowerCase() === 'facebook' && !user.email) {
                    $rootScope.$broadcast('userError', 'missing_email');
                } 

            } 


            //Logged out states
            if (!user.id || user.anonymous) {

                //Trigger FTX
                storage
                .get(['ftxLeft', 'showedBundles'])
                .then(function(res) {


                    //Skip auth experiment
                    if (user.allowAnonymous && res.showedBundles && !user.hasUserLogin) {

                        //Set FTX dialogs still left unseen
                        $rootScope.vm.ftxLeft = res.ftxLeft || [];

                        //Set FTX reminder for Most-visited if it's not seen yet
                        var tomorrow = new Date($rootScope.installDate);
                        tomorrow.setDate(tomorrow.getDate() + 1);

                        if (tomorrow < new Date() && $rootScope.vm.ftxLeft.indexOf('most-visited') !== -1) {                         
                            $rootScope.showFtx('most-visited')
                        }


                        return;
                    }   

                    //Force sign in if user already completed FTX
                    if (res.showedBundles) {
                        $state.go('sign-in');
                        return;
                    }

                    //Engage FTX
                    if (!res.showedBundles) {

                        $rootScope.vm.ftx = true;

                        if (window.isMuzliSafari) {
                            $state.go('safari')
                        } else {
                            $state.go('welcome')
                        }   

                    }
                });

            }

        })



        /*======================================
        =            User functions            =
        ======================================*/

        $rootScope.shareAfterLogin = function(channel) {

            socialService.share(channel, {
                'twitter': 'http://bit.ly/1MpxFig',
                'facebook': 'http://bit.ly/1qj6Em5',
                'linkedin': 'http://bit.ly/1Q32LqE'
            }[channel], "Lovin' Muzli! Design inspiration on tap. Check it out.");

            trackService.track({
                category: 'Share Promote Dialog',
                action: channel,
                label: 'https://muz.li/'
            });
        };

        $rootScope.clickOutsideBubble = function($event) {
            if ($rootScope.userError) {
                $rootScope.hideUserError = true;
            }
        };

        $rootScope.clickUser = function() {
            
            var user = $rootScope.user;

            if (user.provider && user.provider.toLowerCase() === 'facebook' && !user.email) {
                userService.rerequest();
            } else {
                $state.go('favorites', {}, { reload: true });
            }
        };

        $rootScope.logOut = function() {
            userService.logOut();
        };

        $rootScope.loginFacebook = function(from) {

           if (from) {
                trackService.track({
                    category: 'SignIn',
                    action: from,
                    label: 'facebook'
                });
            }

            userService.login('facebook');
        };

        $rootScope.loginTwitter = function(from) {

            if (from) {
                trackService.track({
                    category: 'SignIn',
                    action: from,
                    label: 'twitter'
                });
            }

            userService.login('twitter');
        };

        $rootScope.loginGoogle = function(from) {

            if (from) {
                trackService.track({
                    category: 'SignIn',
                    action: from,
                    label: 'google'
                });
            }

            userService.login('google');
        };

        function syncLocalToUserData() {

            userService
            .getData()
            .then((user) => {

                if (!user) {
                    user = {}
                }

                $('body').removeClass('theme');

                storage.get(['enableSharebleLinks', 'enablePalettes']).then((storageData) => {

                    if (user.enableSharebleLinks !== undefined) {
                        $rootScope.enableSharebleLinks = user.enableSharebleLinks
                    } else {
                        $rootScope.enableSharebleLinks = storageData.enableSharebleLinks
                    }

                    if (user.enablePalettes !== undefined) {
                        $rootScope.enablePalettes = user.enablePalettes
                    } else {
                        $rootScope.enablePalettes = storageData.enablePalettes
                    }
    
                    //If no enableSharebleLinks is set, default to true;
                    if ($rootScope.enableSharebleLinks === undefined || $rootScope.enableSharebleLinks === null) {
                        $rootScope.enableSharebleLinks = true;
                        localStorage.enableSharebleLinks = true;
                    }
    
                    //If no enablePalettes is set, default to true;
                    if ($rootScope.enablePalettes === undefined || $rootScope.enablePalettes === null) {
                        localStorage.enablePalettes = true;
                        $rootScope.enablePalettes = true;
                    }

                })

                $rootScope.theme = user.theme || 'white';
                $rootScope.timeFormat = user.timeFormat || '12h';
                
                var update = {};

                if (user.halfView != null) {
                    update.halfView = user.halfView;
                }

                if (user.theme != null) {
                    update.theme = user.theme;
                }

                if (user.topSitesDisabled != null) {
                    update.topSitesDisabled = user.topSitesDisabled;
                }

                storage.set(update).then(function() {

                    if (update.halfView != null) {
                        $rootScope.$broadcast('muzliSetView');
                    }

                    if (update.topSitesDisabled != null) {
                        $rootScope.$broadcast('muzliLoadSites');
                    }
                })

            });
        }

        syncLocalToUserData();
    }

    angular.module('user', []).config(config).run(run);

    angular.module('user').controller('emailAuthController', ['$scope', '$rootScope', 'trackService', 'userService',
        
        function ($scope, $rootScope, trackService, userService) {

            function validateEmail(email) {
                const re = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
                return re.test(String(email).toLowerCase());
            }

            $scope.state = $rootScope.user.hasUserLogin ? 'sign-in' : 'sign-up';

            $scope.signUpModel = {
                email: '',
                name: '',
                verification: [],
                sendNewsletter: true,
            }

            $scope.isLoading = false;

            $scope.switchState = function (state, $event) {

                if ($event) {
                    $event.preventDefault();
                    $event.stopPropagation();
                }

                $scope.state = state;
            }

            $scope.emailSignUp = function() {

                trackService.track({
                    category: 'SignIn',
                    action: 'Form Submit',
                });

                $scope.doesNotExist = false;

                if (!$scope.signUpModel.email) {
                    $scope.emptyError = true;
                    return;
                }

                if (!validateEmail($scope.signUpModel.email)) {
                    $scope.badEmail = true;
                    return;
                }

                //Set state to figure out which action user took 
                $scope.signUpModel.action = $scope.state;

                userService
                .loginEmail($scope.signUpModel)
                .then((data) => {
                        
                    if (data.error === 'not_exist') {
                        $scope.doesNotExist = true;
                        return;
                    }

                    //If no error returned, jump to verify state
                    $scope.state = 'verify';


                    // if (item.badLink) {
                    //     $scope.isLoading = false;
                    //     $scope.badUrl = true;
                    // }

                    // if (item.dribbbleProhibited) {
                    //     $scope.isLoading = false;
                    //     $scope.dribbbleProhibited = true;
                    // }

                    // if (item.itemAlreadyExists) {
                    //     $scope.isLoading = false;
                    //     $scope.alreadyExist = true;
                    //     $scope.existingUrl = MUZLI_SEARCH_URL + '/' + btoa(item.itemId).slice(0,10);
                    // }

                    // if (item.id) {
                    //     window.location.href = MUZLI_SEARCH_URL + '/' + btoa(item.id).slice(0,10) + '?success=true';
                    // } else {
                    //     $scope.isLoading = false;
                    // }

                })
            };

            $scope.emailVerify = function(link) {

                trackService.track({
                    category: 'SignIn',
                    action: 'Form Verify',
                });

                $scope.tooManyAttempts = false;
                $scope.badVerification = false;

                if (!$scope.signUpModel.email) {
                    $scope.emptyError = true;
                    return;
                }

                if ($scope.signUpModel.verification.length !== 6) {
                    $scope.badVerification = true;
                    return;
                }

                userService
                .verifyEmail($scope.signUpModel)
                .then((data) => {

                    if (data.error === 'bad_verification') {
                        $scope.badVerification = true;
                        return;
                    }

                    if (data.error === 'too_many_attempts') {
                        $scope.tooManyAttempts = true;
                        $scope.badVerification = false;
                        return;
                    }

                    if (data.status === 'ok' && data.redirect) {
                        window.location = data.redirect;
                        return;
                    }

                })
            };

            $scope.emailResend = function($event) {

                if ($event) {
                    $event.preventDefault();
                    $event.stopPropagation();
                }

                trackService.track({
                    category: 'SignIn',
                    action: 'Form Resend',
                });

                if (!$scope.signUpModel.email) {
                    $scope.emptyError = true;
                    return;
                }

                if (!validateEmail($scope.signUpModel.email)) {
                    $scope.badEmail = true;
                    return;
                }

                userService
                .loginEmail($scope.signUpModel)
                .then((data) => {

                    if (data.status === 'ok') {

                       $scope.signUpModel.isResending = true;

                       setTimeout(() => {
                           $scope.signUpModel.isResending = false;
                       }, 15 * 1000);

                    }

                })
            };


    }]);


})();
